package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.task.service.ExamBossService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description 清理考试记录缓存
 * @Author lideyin
 * @Date 2019/12/24 18:07
 * @Version 1.0
 */
@Component
public class ClearExamDataCacheExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamBossService examBossService;

    /**
     * 执行
     *
     * @param key
     * @param examRecordData
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     * @author WANGWEI
     */
    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        Boolean removable, TaskContext context) throws Exception {

        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + examRecordData.getStudentId();

        try {
            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            //只有已同步成功的数据，才执行清理操作
            if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {
                clearExamCache(examRecordData.getExamStudentId());
            }

        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
        }
    }

    /**
     * 考试id
     *
     * @param examId
     */
    private void clearExamCache(Long examId) {
        ExamBoss examBoss = examBossService.getExamBoss(examId);

        if (null == examBoss) {
            return;
        }

        //如果开考次数==考试完结次数，则删除考试基础信息缓存
        if (examBoss.getStartCount() == examBoss.getEndCount()) {
            examBossService.deleteExamBoss(examId);
        }
    }

}
